﻿using FccLite.Web.Domain.FccNozzleInfo;
using FccLite.Web.Domain.FccOilInfo;
using FccLite.Web.Domain.FccStationInfo;
using FccLite.Web.Domain.FccTankInfo;
using Microsoft.EntityFrameworkCore;

namespace FccLite.Web.utils.database
{
    public class MysqlDbContext:DbContext
    {
        public MysqlDbContext(DbContextOptions<MysqlDbContext> options):base(options) { }

        public DbSet<FccStationInfo> FccStationInfos { get; set; }

        public DbSet<FccOilInfo> OilInfos { get; set; }

        public DbSet<FccTankInfo> TankInfos { get; set; }

        public DbSet<FccNozzleInfo> NozzleInfos { get; set; }

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            modelBuilder.Entity<FccTankInfo>()
                .HasOne(tankInfos => tankInfos.Oil)
                .WithMany(oilInfo => oilInfo.FccTankInfo)
                .HasForeignKey(tankInfo => tankInfo.OilId)
                .OnDelete(DeleteBehavior.Restrict);
                
            modelBuilder.Entity<FccNozzleInfo>()
                .HasOne(nozzleInfo => nozzleInfo.Tank)
                .WithMany(tankInfo => tankInfo.FccNozzleInfos)
                .HasForeignKey(nozzleInfo => nozzleInfo.TankId)
                .OnDelete(DeleteBehavior.Restrict);

            base.OnModelCreating(modelBuilder);
        }
    }
}
