﻿using System.Numerics;

namespace FccLite.Web.Domain.FccStationInfo.Input
{
    public class StationInfoInput
    {
        /// <summary>
        /// id
        /// </summary>
        public long? Id { get; set; }

        /// <summary>
        /// 云端站点 id
        /// </summary>
        public string? BuildId { get; set; }

        /// <summary>
        /// 站点名称
        /// </summary>
        public string? Name { get; set; }

        /// <summary>
        /// 经度
        /// </summary>
        public decimal? Longitude { get; set; }

        /// <summary>
        /// 纬度
        /// </summary>
        public decimal? Latitude { get; set; }

        /// <summary>
        /// 小程序链接地址
        /// </summary>
        public string? SmallProgram { get; set; }

        /// <summary>
        /// 云端地址
        /// </summary>
        public string? CloudService { get; set; }

        /// <summary>
        /// 云端  MQTT 地址
        /// </summary>
        public string? MqttService { get; set; }

        /// <summary>
        /// icard 数据库连接地址
        /// </summary>
        public string? IcardService { get; set; }

        /// <summary>
        /// 本地 fcc 与油机 socket 通讯端口
        /// </summary>
        public string? ServicePort { get; set; }

        /// <summary>
        /// 本地 fcc 与 fcc 页面 webSocket 端口
        /// </summary>
        public string? WebSocketPort { get; set; }

        public FccStationInfo? ToComponent()
        {
            if (IsNotNorm()) return null;
            
            return new FccStationInfo()
            {
                BuildId = this.BuildId,
                Name = this.Name != null ? this.Name : "",
                Longitude = (decimal)(this.Longitude != null ? this.Longitude : new decimal(0)),
                Latitude = (decimal)(this.Latitude != null ? this.Latitude : new decimal(0)),
                SmallProgram = this.SmallProgram,
                CloudService = this.CloudService,
                MqttService = this.MqttService,
                IcardService = this.IcardService != null ? this.IcardService : "",
                ServicePort = this.ServicePort,
                WebSocketPort = this.WebSocketPort
            };
        }

        public Boolean IsNotNorm()
        {
            return string.IsNullOrEmpty(this.BuildId) || !this.Longitude.HasValue || !this.Latitude.HasValue || string.IsNullOrEmpty(this.SmallProgram) || string.IsNullOrEmpty(this.CloudService)
                || string.IsNullOrEmpty(this.MqttService) || string.IsNullOrEmpty(this.ServicePort) || string.IsNullOrEmpty(this.WebSocketPort);
        }
    }
}
