﻿using FccLite.Web.Domain.FccNozzleInfo.Input;
using FccLite.Web.utils.database;
using Microsoft.EntityFrameworkCore;

namespace FccLite.Web.Repositories.FccNozzleInfo
{
    public class NozzleInfoReposity : INozzleInfoReposity
    {
        static NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        private readonly MysqlDbContext _dbContext;

        public NozzleInfoReposity(MysqlDbContext dbContext)
        {
            _dbContext = dbContext;
        }

        /// <summary>
        /// 根据油枪 id 找油枪信息
        /// </summary>
        /// <param name="id">油枪id</param>
        /// <returns></returns>
        public async Task<Domain.FccNozzleInfo.FccNozzleInfo?> GetNozzleInfoById(long id)
        {
            return await _dbContext.NozzleInfos.FirstOrDefaultAsync(info => info.Id == id);
        }

        /// <summary>
        /// 新增、修改油枪信息
        /// </summary>
        /// <param name="uploadNozzleInfoInput">油枪信息</param>
        /// <returns></returns>
        public async Task<int> UploadNozzleInfo(UploadNozzleInfoInput uploadNozzleInfoInput)
        {
            if(uploadNozzleInfoInput.Id != null)
            {
                Domain.FccNozzleInfo.FccNozzleInfo? fccNozzleInfo = await _dbContext.NozzleInfos.FirstOrDefaultAsync(info => info.Id == uploadNozzleInfoInput.Id);
                if(fccNozzleInfo == null)
                {
                    Logger.Info($"find nozzle info with id : {uploadNozzleInfoInput.Id} for update, result is null");
                    return 0;
                }
                fccNozzleInfo.MachineIp = uploadNozzleInfoInput.Ip;
                fccNozzleInfo.FuelPoint = uploadNozzleInfoInput.FuelPoint;
                fccNozzleInfo.InternalNum = uploadNozzleInfoInput.InternalNum;
                fccNozzleInfo.ExternalNum = uploadNozzleInfoInput.NozzleNum;
                fccNozzleInfo.TankId = uploadNozzleInfoInput.TankId;
                fccNozzleInfo.TankNum = uploadNozzleInfoInput.TankNum;
                
                return await _dbContext.SaveChangesAsync();
            }

            Domain.FccNozzleInfo.FccNozzleInfo info = uploadNozzleInfoInput.ToComponent();
            _dbContext.NozzleInfos.Add(info);
            return await _dbContext.SaveChangesAsync();
        }

        /// <summary>
        /// 根据id 删除油枪信息
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<int> DeleteNozzleInfoById(long id)
        {
            Domain.FccNozzleInfo.FccNozzleInfo? fccNozzleInfo = await _dbContext.NozzleInfos.FirstOrDefaultAsync(info => info.Id == id);
            if (fccNozzleInfo == null)
            {
                Logger.Info($"find nozzle info with id : {id} for delete,result is null");
                return 0;
            }

            _dbContext.NozzleInfos.Remove(fccNozzleInfo);
            return await _dbContext.SaveChangesAsync();
        }
    }
}
