﻿using FccLite.Web.Domain.FccNozzleInfo.Input;
using FccLite.Web.Domain.FccNozzleInfo.Output;
using FccLite.Web.Repositories.FccNozzleInfo;
using FccLite.Web.Repositories.FccTankInfo;

namespace FccLite.Web.Services.FccNozzleInfo
{
    public class NozzleInfoServiceImpl : INozzleInfnService
    {
        static NLog.Logger Logger = NLog.LogManager.GetCurrentClassLogger();

        private readonly INozzleInfoReposity _nozzleInfoReposity;
        private readonly ITankReposity _tankReposity;
        public NozzleInfoServiceImpl(INozzleInfoReposity nozzleInfoReposity,ITankReposity tankReposity)
        {
            _nozzleInfoReposity = nozzleInfoReposity;
            _tankReposity = tankReposity;
        }
        /// <summary>
        /// 新增、修改油枪信息
        /// </summary>
        /// <param name="nozzleInfoInput">油枪信息</param>
        /// <returns></returns>
        public async Task<SetNozzleInfoOutput> UploadDetailsNozzleInfo(UploadNozzleInfoInput nozzleInfoInput)
        {
            int row = await _nozzleInfoReposity.UploadNozzleInfo(nozzleInfoInput);
            Logger.Info($"upload nozzle info ,chage row {row}");
            if(row > 0)
            {
                return new SetNozzleInfoOutput()
                {
                    Result = true,
                    Message = "保存成功"
                };
            }

            return new SetNozzleInfoOutput()
            {
                Result = false,
                Message = "保存失败"
            };
        }

        /// <summary>
        /// 根据id获取油枪信息
        /// </summary>
        /// <param name="id">油枪id</param>
        /// <returns></returns>
        public async Task<DetailsNozzleInfoOutput?> GetDetailsNozzleInfoById(long id)
        {
            Domain.FccNozzleInfo.FccNozzleInfo? fccNozzleInfo = await _nozzleInfoReposity.GetNozzleInfoById(id);
            if (fccNozzleInfo == null)
            {
                Logger.Info($"find nozzle Info with id {id} for details ,result is null");
                return null;
            }
            List<DetailsNozzleInfoOutput.RelatableTank> tanks = await _tankReposity.GetTankInfo();
            Logger.Info($"find all tanks info for nozzle info details,count is {tanks.Count}");

            return new DetailsNozzleInfoOutput()
            {
                Id = fccNozzleInfo.Id,
                NozzleNum = fccNozzleInfo.ExternalNum,
                Ip = fccNozzleInfo.MachineIp,
                FuelPoint = fccNozzleInfo.FuelPoint,
                InternalNum = fccNozzleInfo.InternalNum,
                TankId = fccNozzleInfo.TankId,
                TankNum = fccNozzleInfo.TankNum,
                RelatableTanks = tanks,
            };
            
        }



        /// <summary>
        /// 根据油枪id删除油枪信息
        /// </summary>
        /// <param name="id">油枪id</param>
        /// <returns></returns>
        public async Task<SetNozzleInfoOutput> DeleteNozzleInfoByID(long id)
        {
            int row = await _nozzleInfoReposity.DeleteNozzleInfoById(id);
            Logger.Info($"delete nozzle info ,chage row {row}");
            if (row > 0)
            {
                return new SetNozzleInfoOutput()
                {
                    Result = true,
                    Message = "删除成功"
                };
            }

            return new SetNozzleInfoOutput()
            {
                Result = false,
                Message = "删除失败"
            };
        }
    }
}
